require 'faraday'
require 'uri'
require 'google_doc_seed'
require 'json'
require 'csv'

module Hyrb
  module Tasks
    module Developers
      class Inject < Task
        def run(env)
          env.developers = Hyrb::Models::Cache::Developers.new.data
        end
      end

      class Init < Task
        depends Inject

        def run(env)
          invoke(Download) if env.developers.blank?
        end
      end

      class Show < Task
        depends Init

        def run(env)
          say env.developers.map(&:to_hash).to_yaml
        end
      end

      class Download < Task
        depends Google::Init

        def run(env)
          response = Faraday.post "https://accounts.google.com/o/oauth2/token",
            refresh_token: env.creds.google_refresh_token,
            client_id:     env.creds.google_client_id,
            client_secret: env.creds.google_client_secret,
            grant_type:    "refresh_token"

          access_token = JSON.parse(response.body)['access_token']
          seeder = GoogleDocSeed.new(access_token)
          csv_string = seeder.to_csv_string(env.creds.google_spreadsheet_key)

          csv = CSV.parse(csv_string, GoogleCSVConverters::CSV_SETTINGS)

          env.developers = Hyrb::Models::Cache::Developers.new
          env.developers.data = csv.map do |row|
            pks = []
            while true
              field = row.delete(:public_key)
              if field.length > 1
                pks << field[1] if field[1]
              else
                break
              end
            end

            Hyrb::Models::Developer.new({
              name:            row[:name],
              email:           row[:email],
              role:            Hyrb::Models::Developer::ROLE_MAP[row[:role].to_i],
              github_username: row[:github],
              keys:            pks,
            }) if row[:role].to_i < 4 && row[:email]
          end.compact

          env.developers.save!
          say "Developer data downloaded", :green
        end
      end

      class AddToProject < Task
        depends Init, Project::Init

        def run(env)
          added_devs = if env.project.users.try(:any?)
            env.developers.select {|dev| env.project.users.include? dev.email }
          else
            env.developers.select {|dev| dev.role == :admin }
          end

          loop do
            say "Devs with access"
            list_devs(added_devs)

            break unless yes? "Add more devs?"

            dev_list = env.developers - added_devs
            devs = *prompt_for_dev(dev_list)
            added_devs += devs
          end

          env.project.users = added_devs.map(&:email)
          env.project.save!
        end

        def list_devs(devs)
          devs.each {|d| say "#{d.name} <#{d.email}>"}
        end

        def prompt_for_dev(devs)
          option_list(devs) { |d, i| "#{i+1}: #{d.name} <#{d.email}>" }
        end
      end

      module GoogleCSVConverters
        BLANK_TO_NIL = ->(f) { f.empty? ? nil : f }
        TRIM_WHITESPACE = ->(f) { f.respond_to?(:gsub) ? f.gsub(/\s+$/, '') : f }

        CSV_SETTINGS = {
          headers:           true,
          header_converters: :symbol,
          converters:        [BLANK_TO_NIL, TRIM_WHITESPACE],
        }
      end
    end
  end
end