require 'fog/digitalocean'
require 'parallel'
  
module Hyrb
  module Tasks
    module DigitalOcean
      class Inject < Task
        depends Creds::Init

        prompt :creds, :digital_ocean_api_key
        prompt :creds, :digital_ocean_client_id

        def run(env)
          env.digital_ocean_client = Fog::Compute.new({
            provider: 'DigitalOcean',
            digitalocean_api_key: env.creds.digital_ocean_api_key,
            digitalocean_client_id: env.creds.digital_ocean_client_id,
          })
        end
      end

      class Init < Task
        depends Inject

        def run(env)
          # TODO: allow cache referesh to be forced
          env.digital_ocean_cache = Hyrb::Models::Cache::DigitalOcean.new
          env.digital_ocean_cache.class.keys.each do |key|
            env.digital_ocean_cache[key] ||= env.digital_ocean_client.send(key).map(&:attributes)
          end
          env.digital_ocean_cache.save!
        end
      end

      class SSHKeys < Task
        depends Init, Developers::Init

        def run(env)
          env.digital_ocean_ssh_keys = Parallel.map(env.digital_ocean_client.ssh_keys.all, in_threads: 12) do |dokey|
            env.digital_ocean_client.ssh_keys.get(dokey.id)
          end

          env.digital_ocean_ssh_keys.each do |k|
            dev = env.developers.find { |dev| dev.keys.include? k.ssh_pub_key }
            dev.digital_ocean_id = k.id if dev
          end
        end
      end

      class ShowSSHKeys < Task
        depends SSHKeys

        def run(env)
          say env.digital_ocean_ssh_keys.map { |k| {name: k.name, id: k.id, ssh_key: k.ssh_pub_key} }.to_yaml
        end
      end

      class SyncSSHKeys < Task
        depends SSHKeys

        def run(env)
          env.digital_ocean_ssh_keys.each do |dokey|
            unless key = env.developers.any? { |dev| dev.keys.include? dokey.ssh_pub_key }
              say "Removing #{dokey.name} SSH key from Digital Ocean", :green
              env.digital_ocean_client.ssh_keys.destroy(dokey.id)
            end
          end

          env.developers.select { |d| d.keys.any? }.each do |dev|
            if key = env.digital_ocean_ssh_keys.any? { |k| dev.keys.include? k.ssh_pub_key }
              say "#{dev.name}'s keys already added to Digital Ocean", :yellow
            else
              dev.keys.each do |pubkey, i|
                say "Adding pubkey for #{dev.name} to Digital Ocean", :green
                env.digital_ocean_client.ssh_keys.create(name: dev.name, ssh_pub_key: pubkey)
              end
            end
          end
        end
      end
    
    end
  end
end