require 'octokit'

module Hyrb
  module Tasks
    module Github
      class Inject < Task
        depends Creds::Init

        prompt :creds, :github_org
        prompt :creds, :github_api_key

        def run(env)
          env.github_client = Octokit::Client.new({
            access_token: env.creds.github_api_key,
            auto_pagination: true,
          })
        end
      end

      class Init < Task
        depends Inject
      end

      class Repos < Task
        depends Init

        def run(env)
          env.github_repos = env.github_client.org_repos(per_page: 100, org: env.creds.github_org)
        end
      end

      class ShowRepos < Task
        depends Repos

        def run(env)
          say env.github_repos.map(&:name).to_yaml
        end
      end

      class CreateRepo < Task
        depends Init, Project::Init

        prompt :project, :github_org, default: ->(env) { env.creds.github_org }
        prompt :project, :repo_name, default: ->(env) { env.project.name }

        def run(env)
          if env.github_repo = env.github_client.repository?(env.project.repo_path)
            say "Github repo #{env.project.repo_path} exists", :yellow
          else
            # TODO: gitignore, teams
            env.github_repo = env.github_client.create(env.project.repo_name, {
              organization: env.project.github_org,
              description: env.project.description,
              private: true,
            })
            say "Created Github repo #{env.project.repo_path}", :green
          end
        end
      end

      class CreateProjectTeam < Task
        depends CreateRepo, Developers::Init

        prompt :project, :github_team, default: ->(env) { env.project.repo_name }

        def run(env)
          env.github_team = env.github_client.organization_teams(env.project.github_org).find do |t|
            t.name == env.project.github_team
          end

          if env.github_team
            say "Github team #{env.project.github_team} exists", :yellow
          else
            env.github_team = env.github_client.create_team(env.project.github_org, {
              name: env.project.github_team,
              repo_names: [env.project.repo_name],
              permission: "push"
            })
            say "Created Github team #{env.project.github_team} exists", :green
          end
        end
      end

      class SyncProjectTeam < Task
        depends CreateProjectTeam

        def run(env)
          team = env.github_team
          new_devs = env.project.developers(env.developers)
          current_members = env.github_client.team_members(team.id)

          members_to_remove = current_members.reject do |member|
            new_devs.map(&:github_username).include?(member.login)
          end

          devs_to_add = new_devs.reject do |dev|
            current_members.map(&:login).include?(dev.github_username)
          end

          members_to_remove.each do |member|
            if env.github_client.remove_team_member(team.id, member.login)
              say "Removed #{member.login} from team #{env.project.github_org}/#{team.slug}", :green
            else
              say "Couldn't remove #{member.login} from team #{env.project.github_org}/#{team.slug}", :red
            end
          end

          devs_to_add.each do |dev|
            if env.github_client.add_team_member(team.id, dev.github_username)
              say "Added #{dev.github_username} to team #{env.project.github_org}/#{team.slug}", :green
            else
              say "Couldn't add #{dev.github_username} to team #{env.project.github_org}/#{team.slug}", :red
            end
          end
        end
      end

    end
  end
end