require 'ipaddr'

require 'parallel'
require 'fog/rackspace'

module Hyrb
  module Tasks
    module Rackspace
      class Inject < Task
        depends Creds::Init

        prompt :creds, :rackspace_username
        prompt :creds, :rackspace_api_key
        prompt :defaults, :rackspace_region, default: ->(env) { env.rackspace_region }

        def run(env)
          env.rackspace_client = Fog::Compute.new({
            provider: 'rackspace',
            rackspace_username: env.creds.rackspace_username,
            rackspace_api_key: env.creds.rackspace_api_key,
            version: :v2,
            rackspace_region: env.defaults.rackspace_region,
          })

          env.rackspace_dns_client = Fog::DNS.new({
            provider: 'rackspace',
            rackspace_username: env.creds.rackspace_username,
            rackspace_api_key: env.creds.rackspace_api_key,
            rackspace_region: env.defaults.rackspace_region,
          })
        end
      end

      class Init < Task
        depends Inject

        def run(env)
          env.rackspace_cache = Hyrb::Models::Cache::Rackspace.new
          env.rackspace_cache.class.keys.each do |key|
            env.rackspace_cache[key] ||= env.rackspace_client.send(key).map(&:attributes)
          end
          env.rackspace_cache.save!
        end
      end

      class DNSZones < Task
        depends Init

        def run(env)
          env.rackspace_dns_zones = env.rackspace_dns_client.zones
        end
      end

      class CreateDNSRecord < Task
        depends DNSZones, Environment::Init

        prompt :environment, :domain, default: ->(env) { "#{env.environment.label}.#{env.defaults.domain}" }
        prompt :environment, :host

        def run(env)
          record_type = (IPAddr.new(env.environment.host) rescue false) ? 'A' : 'CNAME'

          # assume zone is going to be x.com
          proj_zone = env.environment.domain.split('.')[-2..-1].join('.')

          zone = env.rackspace_dns_zones.find { |z| z.domain == proj_zone }
          zone ||= env.rackspace_dns_client.zones.create({
            domain: proj_zone,
            email: env.defaults.admin_email,
          })

          record = zone.records.find do |r|
            r.name == env.environment.domain && r.type == record_type
          end

          if record
            say "Record exists: '#{record.type} #{record.name} #{record.value} (#{record.ttl})'", :yellow
          else
            record = zone.records.create({
              name: env.environment.domain,
              value: env.environment.host,
              type: record_type,
              ttl: 300,
            })
            say "Created record: '#{record.type} #{record.name} #{record.value} (#{record.ttl})'", :green
          end
          env.environment.has_dns_record = true
          env.environment.save!
          beep
        end
      end

    end
  end
end